@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2008 Red Hat Inc.
+// Copyright (C) 2008-2009 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -12,6 +12,7 @@
 #include <cstring>
 #include <string>
 #include <elfutils/libdwfl.h>
+#include <dwarf.h>
 
 using std::string;
 
@@ -21,7 +22,7 @@ void dwfl_assert(const string& desc, int rc)
     return;
   string msg = "libdwfl failure (" + desc + "): ";
   if (rc < 0)
-    msg += dwfl_errmsg (rc);
+    msg += (dwfl_errmsg (rc) ?: "?");
   else
     msg += std::strerror (rc);
   throw semantic_error (msg);
@@ -42,5 +43,19 @@ void dwarf_assert(const string& desc, int rc)
 void dwfl_assert(const std::string& desc, bool condition)
 {
     if (!condition)
-        dwarf_assert(desc, -1);
+        dwfl_assert(desc, -1);
 }
+
+
+// Helper for dealing with selected portions of libdwfl in a more readable
+// fashion, and with specific cleanup / checking / logging options.
+
+const char *
+dwarf_diename_integrate (Dwarf_Die *die)
+{
+  Dwarf_Attribute attr_mem;
+  return dwarf_formstring (dwarf_attr_integrate (die, DW_AT_name, &attr_mem));
+}
+
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
