@@ -7,7 +7,7 @@
  * Public License (GPL); either version 2, or (at your option) any
  * later version.
  *
- * Copyright (C) 2005-2008 Red Hat Inc.
+ * Copyright (C) 2005-2009 Red Hat Inc.
  */
 
 #include "staprun.h"
@@ -318,6 +318,41 @@ int init_stapio(void)
   if (target_cmd)
     start_cmd();
 
+  /* Run in background */
+  if (daemon_mode) {
+    pid_t pid;
+    int ret;
+    dbug(2, "daemonizing stapio\n");
+
+    /* daemonize */
+    ret = daemon(0, 1); /* don't close stdout at this time. */
+    if (ret) {
+      err("Failed to daemonize stapio\n");
+      return -1;
+    }
+
+    /* change error messages to syslog. */
+    switch_syslog("stapio");
+
+    /* show new pid */
+    pid = getpid();
+    fprintf(stdout, "%d\n", pid);
+    fflush(stdout);
+
+    /* redirect all outputs to /dev/null */
+    ret = open("/dev/null", O_RDWR);
+    if (ret < 0) {
+      err("Failed to open /dev/null\n");
+      return -1;
+    }
+    close(STDIN_FILENO);
+    close(STDOUT_FILENO);
+    close(STDERR_FILENO);
+    dup2(ret, STDOUT_FILENO);
+    dup2(ret, STDERR_FILENO);
+    close(ret);
+  }
+
   return 0;
 }
 
@@ -360,10 +395,10 @@ void cleanup_and_exit(int detach)
 #define BUG9788_WORKAROUND
 #ifndef BUG9788_WORKAROUND
     dbug(2, "removing %s\n", modname);
-    if (execlp(staprun, basename (staprun), "-d", modname, NULL) < 0) {
+    if (execlp(staprun, basename (staprun), "-d", modpath, NULL) < 0) {
       if (errno == ENOEXEC) {
 	char *cmd;
-	if (asprintf(&cmd, "%s -d '%s'", staprun, modname) > 0)
+	if (asprintf(&cmd, "%s -d '%s'", staprun, modpath) > 0)
 	  execl("/bin/sh", "sh", "-c", cmd, NULL);
 	free(cmd);
       }
@@ -392,10 +427,10 @@ void cleanup_and_exit(int detach)
 
     if (pid == 0) {			/* child process */
       /* Run the command. */
-      if (execlp(staprun, basename (staprun), "-d", modname, NULL) < 0) {
+      if (execlp(staprun, basename (staprun), "-d", modpath, NULL) < 0) {
 	if (errno == ENOEXEC) {
 	  char *cmd;
-	  if (asprintf(&cmd, "%s -d '%s'", staprun, modname) > 0)
+	  if (asprintf(&cmd, "%s -d '%s'", staprun, modpath) > 0)
 	    execl("/bin/sh", "sh", "-c", cmd, NULL);
 	  free(cmd);
 	}
@@ -454,21 +489,14 @@ int stp_main_loop(void)
     switch (type) {
 #ifdef STP_OLD_TRANSPORT
     case STP_REALTIME_DATA:
-      {
-        ssize_t bw = write(out_fd[0], data, nb);
-        if (bw >= 0 && bw != nb) {
-          nb = nb - bw;
-          bw = write(out_fd[0], data, nb);
-        }
-        if (bw != nb) {
-          _perr("write error (nb=%ld)", (long)nb);
-          cleanup_and_exit(0);
-        }
-        break;
+      if (write_realtime_data(data, nb)) {
+        _perr("write error (nb=%ld)", (long)nb);
+        cleanup_and_exit(0);
       }
+      break;
 #endif
     case STP_OOB_DATA:
-      fputs((char *)data, stderr);
+      eprintf("%s", (char *)data);
       break;
     case STP_EXIT:
       {
@@ -477,6 +505,14 @@ int stp_main_loop(void)
         cleanup_and_exit(0);
         break;
       }
+    case STP_REQUEST_EXIT:
+      {
+        /* module asks us to start exiting, so send STP_EXIT */
+        dbug(2, "got STP_REQUEST_EXIT\n");
+        int32_t rc, btype = STP_EXIT;
+        rc = write(control_channel, &btype, sizeof(btype));
+        break;
+      }
     case STP_START:
       {
         struct _stp_msg_start *t = (struct _stp_msg_start *)data;
