@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2008 Red Hat Inc.
+// Copyright (C) 2005-2009 Red Hat Inc.
 // Copyright (C) 2006 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -231,6 +231,7 @@ struct target_symbol: public symbol
     };
   std::string base_name;
   std::vector<std::pair<component_type, std::string> > components;
+  std::string probe_context_var;
   semantic_error* saved_conversion_error;
   target_symbol(): saved_conversion_error (0) {}
   void print (std::ostream& o) const;
@@ -238,6 +239,15 @@ struct target_symbol: public symbol
 };
 
 
+struct cast_op: public target_symbol
+{
+  expression *operand;
+  std::string type, module;
+  void print (std::ostream& o) const;
+  void visit (visitor* u);
+};
+
+
 struct arrayindex: public expression
 {
   std::vector<expression*> indexes;
@@ -680,6 +690,7 @@ struct visitor
   virtual void visit_print_format (print_format* e) = 0;
   virtual void visit_stat_op (stat_op* e) = 0;
   virtual void visit_hist_op (hist_op* e) = 0;
+  virtual void visit_cast_op (cast_op* e) = 0;
 };
 
 
@@ -720,6 +731,7 @@ struct traversing_visitor: public visitor
   void visit_print_format (print_format* e);
   void visit_stat_op (stat_op* e);
   void visit_hist_op (hist_op* e);
+  void visit_cast_op (cast_op* e);
 };
 
 
@@ -759,6 +771,7 @@ struct varuse_collecting_visitor: public functioncall_traversing_visitor
   void visit_pre_crement (pre_crement *e);
   void visit_post_crement (post_crement *e);
   void visit_foreach_loop (foreach_loop *s);
+  void visit_cast_op (cast_op* e);
 
   bool side_effect_free ();
   bool side_effect_free_wrt (const std::set<vardecl*>& vars);
@@ -808,6 +821,7 @@ struct throwing_visitor: public visitor
   void visit_print_format (print_format* e);
   void visit_stat_op (stat_op* e);
   void visit_hist_op (hist_op* e);
+  void visit_cast_op (cast_op* e);
 };
 
 // A visitor similar to a traversing_visitor, but with the ability to rewrite
@@ -868,6 +882,7 @@ struct update_visitor: public visitor
   virtual void visit_print_format (print_format* e);
   virtual void visit_stat_op (stat_op* e);
   virtual void visit_hist_op (hist_op* e);
+  virtual void visit_cast_op (cast_op* e);
 
 private:
   std::stack<void *> targets;
@@ -922,6 +937,7 @@ struct deep_copy_visitor: public update_visitor
   virtual void visit_print_format (print_format* e);
   virtual void visit_stat_op (stat_op* e);
   virtual void visit_hist_op (hist_op* e);
+  virtual void visit_cast_op (cast_op* e);
 };
 
 #endif // STAPTREE_H
