@@ -33,6 +33,7 @@ public:
 
   std::ostream& newline (int indent = 0);
   void indent (int indent = 0);
+  void assert_0_indent () { assert (tablevel == 0); }
   std::ostream& line();
 
   std::ostream::pos_type tellp() { return o.tellp(); }
@@ -100,3 +101,5 @@ int translate_pass (systemtap_session& s);
 
 
 #endif // TRANSLATE_H
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
