@@ -8,10 +8,14 @@
 const char *get_home_directory(void);
 int copy_file(const char *src, const char *dest);
 int create_dir(const char *dir);
+int remove_file_or_dir(const char *dir);
 void tokenize(const std::string& str, std::vector<std::string>& tokens,
 	      const std::string& delimiters);
 std::string find_executable(const std::string& name);
 const std::string cmdstr_quoted(const std::string& cmd);
+std::string git_revision(const std::string& path);
+int stap_system(const char *command);
+int kill_stap_spawn(int sig);
 
 
 // stringification generics
